#version 330 core

#define MAX_LIGHT_NUM 6

// Input vertex data, different for all executions of this shader.
layout(location = 0) in vec3 vertexPosition_modelspace;
layout(location = 1) in vec2 vertexUV;
layout(location = 2) in vec3 vertexNormal_modelspace;

// Output data ; will be interpolated for each fragment.
out vec2 UV;
out vec3 Position_worldspace;
out vec3 Normal_cameraspace;
out vec3 EyeDirection_cameraspace;
out vec3 LightDirection_cameraspace[MAX_LIGHT_NUM];
out vec4 lowShadowCoord[MAX_LIGHT_NUM];
out vec4 mediumShadowCoord[MAX_LIGHT_NUM];
out vec4 highShadowCoord[MAX_LIGHT_NUM];

// Values that stay constant for the whole mesh.
uniform mat4 P;
uniform mat4 V;
uniform mat4 M;
uniform bool vShadowApplied;
uniform int vLightNum;
uniform vec3 LightInvDirection_worldspace[MAX_LIGHT_NUM];
uniform mat4 lowDepthBiasMVP[MAX_LIGHT_NUM];
uniform mat4 mediumDepthBiasMVP[MAX_LIGHT_NUM];
uniform mat4 highDepthBiasMVP[MAX_LIGHT_NUM];

uniform float worldScale;

void main(){

	vec3 scaledPosition = vertexPosition_modelspace * worldScale;

	// Output position of the vertex, in clip space : MVP * position
	gl_Position =  P * V * M * vec4(scaledPosition,1);

	if(vShadowApplied)
	{
		for(int lightIndex = 0; lightIndex < vLightNum; lightIndex++){
			lowShadowCoord[lightIndex] = lowDepthBiasMVP[lightIndex] * vec4(scaledPosition,1);
		}

		for(int lightIndex = 0; lightIndex < vLightNum; lightIndex++){
			mediumShadowCoord[lightIndex] = mediumDepthBiasMVP[lightIndex] * vec4(scaledPosition,1);
		}

		for(int lightIndex = 0; lightIndex < vLightNum; lightIndex++){
			highShadowCoord[lightIndex] = highDepthBiasMVP[lightIndex] * vec4(scaledPosition,1);
		}
	}
	
	// Position of the vertex, in worldspace : M * position
	Position_worldspace = (M * vec4(scaledPosition,1)).xyz;
	
	// Vector that goes from the vertex to the camera, in camera space.
	// In camera space, the camera is at the origin (0,0,0).
	EyeDirection_cameraspace = vec3(0,0,0) - ( V * M * vec4(scaledPosition,1)).xyz;

	// Vector that goes from the vertex to the light, in camera space
	for(int lightIndex = 0; lightIndex < vLightNum; lightIndex++){
		LightDirection_cameraspace[lightIndex] = (V*vec4(LightInvDirection_worldspace[lightIndex],0)).xyz;	
	}
	
	// Normal of the the vertex, in camera space
	Normal_cameraspace = ( V * M * vec4(vertexNormal_modelspace,0)).xyz; // Only correct if ModelMatrix does not scale the model ! Use its inverse transpose if not.
	
	// UV of the vertex. No special space for this one.
	UV = vertexUV;
}

